@@ -104,8 +104,12 @@
 #endif
 
 #ifdef HAVE_REGEX_H
+#ifdef HAVE_PCRE2
+#include <pcre2posix.h>
+#else
 #include <regex.h>
 #endif
+#endif
 
 #define MINW 6   /* minimum wait if desired */
 #define CPERS 20 /* # of chars for each sec */
@@ -334,7 +338,7 @@ static char *conv_pat(const char *const orig_str)
     char *new_buf;
 
     size_t cnt = 1; /* allow for '\0' */
-    for (sp = orig_str; *sp != '\0'; sp++)
+    for (sp = orig_str; *sp != '\0'; ++sp)
     {
         const size_t prev_cnt = cnt;
         if (isalpha(*sp))
@@ -408,9 +412,7 @@ static void *do_malloc(const size_t size)
  */
 static FILEDESC *new_fp(void)
 {
-    FILEDESC *fp;
-
-    fp = do_malloc(sizeof *fp);
+    FILEDESC *const fp = do_malloc(sizeof *fp);
     fp->datfd = -1;
     fp->pos = POS_UNKNOWN;
     fp->inf = NULL;
@@ -585,7 +587,6 @@ static int add_file(int percent, const char *file, const char *dir,
     FILEDESC *fp;
     int fd = -1;
     char *path;
-    char *sp;
     struct stat statbuf;
 
     if (!dir)
@@ -623,40 +624,6 @@ static int add_file(int percent, const char *file, const char *dir,
         debugprint("check file fd=%d path=<%s> dir=<%s> file=<%s> percent=%d\n",
             fd, path, dir, file, percent);
         bool found = false;
-        if (!dir && (!strchr(file, '/')))
-        {
-            if (((sp = strrchr(file, '-')) != NULL) && (strcmp(sp, "-o") == 0))
-            {
-#define CALL__add_file(dir) add_file(percent, file, dir, head, tail, parent)
-#define COND_CALL__add_file(loc_dir, dir)                                      \
-    ((!strcmp((loc_dir), (dir))) ? 0 : CALL__add_file(dir))
-                /* BSD-style '-o' offensive file suffix */
-                *sp = '\0';
-                found = CALL__add_file(LOCOFFDIR) ||
-                        COND_CALL__add_file(LOCOFFDIR, OFFDIR);
-                /* put the suffix back in for better identification later */
-                *sp = '-';
-            }
-            else if (All_forts)
-            {
-                found =
-                    (CALL__add_file(LOCFORTDIR) || CALL__add_file(LOCOFFDIR) ||
-                        COND_CALL__add_file(LOCFORTDIR, FORTDIR) ||
-                        COND_CALL__add_file(LOCOFFDIR, OFFDIR));
-            }
-            else if (Offend)
-            {
-                found = (CALL__add_file(LOCOFFDIR) ||
-                         COND_CALL__add_file(LOCOFFDIR, OFFDIR));
-            }
-            else
-            {
-                found = (CALL__add_file(LOCFORTDIR) ||
-                         COND_CALL__add_file(LOCFORTDIR, FORTDIR));
-            }
-#undef COND_CALL__add_file
-#undef CALL__add_file
-        }
         if (!found && !parent && !dir)
         { /* don't display an error when trying language specific files */
             if (env_lang)
@@ -1053,10 +1020,23 @@ static int form_file_list(char **files, int file_cnt)
                 sp = files[i];
             }
         }
+
+        /* BSD-style '-o' offensive file suffix */
+        bool offensive = false;
+        const size_t sp_len = strlen(sp);
+        if (sp_len >= 3 && sp[sp_len - 2] == '-' && sp[sp_len - 1] == 'o')
+        {
+            sp[sp_len - 2] = '\0';
+            offensive = true;
+        }
+
+        const char *fulldir = offensive ? OFFDIR : FORTDIR;
+        const char *locdir = offensive ? LOCOFFDIR : LOCFORTDIR;
+
         if (strcmp(sp, "all") == 0)
         {
-            snprintf(fullpathname, sizeof(fullpathname), "%s", FORTDIR);
-            snprintf(locpathname, sizeof(locpathname), "%s", LOCFORTDIR);
+            snprintf(fullpathname, sizeof(fullpathname), "%s", fulldir);
+            snprintf(locpathname, sizeof(locpathname), "%s", locdir);
         }
         /* if it isn't an absolute path or relative to . or ..
            make it an absolute path relative to FORTDIR */
@@ -1066,9 +1046,8 @@ static int form_file_list(char **files, int file_cnt)
                 strncmp(sp, "../", 3) != 0)
             {
                 snprintf(
-                    fullpathname, sizeof(fullpathname), "%s/%s", FORTDIR, sp);
-                snprintf(
-                    locpathname, sizeof(locpathname), "%s/%s", LOCFORTDIR, sp);
+                    fullpathname, sizeof(fullpathname), "%s/%s", fulldir, sp);
+                snprintf(locpathname, sizeof(locpathname), "%s/%s", locdir, sp);
             }
             else
             {
@@ -1097,7 +1076,7 @@ static int form_file_list(char **files, int file_cnt)
 
                 /* first try full locale */
                 snprintf(
-                    langdir, sizeof(langdir), "%s/%s/%s", FORTDIR, lang, sp);
+                    langdir, sizeof(langdir), "%s/%s/%s", fulldir, lang, sp);
                 ret = add_file(
                     percent, langdir, NULL, &File_list, &File_tail, NULL);
 
@@ -1109,7 +1088,7 @@ static int form_file_list(char **files, int file_cnt)
                     strncpy(ll, lang, 2);
                     ll[2] = '\0';
                     snprintf(
-                        langdir, sizeof(langdir), "%s/%s/%s", FORTDIR, ll, sp);
+                        langdir, sizeof(langdir), "%s/%s/%s", fulldir, ll, sp);
                     ret = add_file(
                         percent, langdir, NULL, &File_list, &File_tail, NULL);
                 }
@@ -1128,8 +1107,19 @@ static int form_file_list(char **files, int file_cnt)
                 ret = add_file(
                     percent, locpathname, NULL, &File_list, &File_tail, NULL);
             }
+            if (strncmp(fullpathname, locpathname, sizeof(fullpathname)) &&
+                strcmp(sp, "all") == 0)
+            {
+                add_file(
+                    percent, locpathname, NULL, &File_list, &File_tail, NULL);
+            }
             if (!ret)
             {
+                if (offensive)
+                {
+                    // restore -o suffix
+                    sp[sp_len - 2] = '-';
+                }
                 snprintf(locpathname, sizeof(locpathname), "%s/%s",
                     getenv("PWD"), sp);
 
@@ -1140,12 +1130,6 @@ static int form_file_list(char **files, int file_cnt)
             {
                 return false;
             }
-            if (strncmp(fullpathname, locpathname, sizeof(fullpathname)) &&
-                strcmp(sp, "all") == 0)
-            {
-                add_file(
-                    percent, locpathname, NULL, &File_list, &File_tail, NULL);
-            }
         }
         else if (!add_file(
                      percent, fullpathname, NULL, &File_list, &File_tail, NULL))
@@ -1174,11 +1158,7 @@ static void getargs(int argc, char **argv)
 #define DEBUG_GETOPT
 #endif
 
-#ifdef NO_OFFENSIVE
-#define OFFENSIVE_GETOPT
-#else
 #define OFFENSIVE_GETOPT "o"
-#endif
 
     while ((ch = getopt(argc, argv,
                 "ac" DEBUG_GETOPT "efhilm:n:" OFFENSIVE_GETOPT "suvw")) != EOF)
@@ -1206,11 +1186,17 @@ static void getargs(int argc, char **argv)
         case 'n':
             SLEN = atoi(optarg);
             break;
-#ifndef NO_OFFENSIVE
         case 'o': /* offensive ones only */
+#ifndef NO_OFFENSIVE
             Offend = true;
-            break;
+#else
+            (void)fprintf(stderr, "%s",
+                "fortune: this fortune was built with -D NO_OFFENSIVE=1"
+                " so the -o flag is unavailable "
+                "(Sorry)\n");
+            exit(1);
 #endif
+            break;
         case 's': /* short ones only */
             Short_only = true;
             Long_only = false;
@@ -1226,13 +1212,13 @@ static void getargs(int argc, char **argv)
                 "(Sorry)\n");
             exit(0);
 #else             /* NO_REGEX */
+        case 'i': /* case-insensitive match */
+            ignore_case = true;
+            break;
         case 'm': /* dump out the fortunes */
             Match = true;
             pat = optarg;
             break;
-        case 'i': /* case-insensitive match */
-            ignore_case = true;
-            break;
 #endif            /* NO_REGEX */
         case 'u': /* Don't recode the fortune */
             No_recode = true;
@@ -1900,7 +1886,7 @@ static void free_desc(FILEDESC *ptr)
             fclose(ptr->inf);
             ptr->inf = NULL;
         }
-        FILEDESC *next = ptr->next;
+        FILEDESC *const next = ptr->next;
         free(ptr);
         ptr = next;
     }
